"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@umijs/utils");
const path_1 = __importDefault(require("path"));
const bundler = (0, utils_1.importLazy)(path_1.default.dirname(require.resolve('@umijs/bundler-webpack/package.json')));
exports.default = async (opts) => {
    for (const config of opts.configProvider.configs) {
        utils_1.logger.info(`Bundle from ${utils_1.chalk.yellow(config.entry)} to ${utils_1.chalk.yellow(path_1.default.join(config.output.path, config.output.filename))}`);
        const { plugins: extraPostCSSPlugins, ...postcssLoader } = config.postcssOptions || {};
        await bundler.build({
            cwd: opts.cwd,
            config: {
                alias: config.alias,
                autoprefixer: config.autoprefixer,
                chainWebpack: config.chainWebpack,
                define: config.define,
                externals: config.externals,
                outputPath: config.output.path,
                // postcss config
                extraPostCSSPlugins,
                postcssLoader,
                // compatible with IE11 by default
                userConfig: {
                    targets: { ie: 11 },
                    jsMinifier: 'terser',
                    cssMinifier: 'cssnano',
                },
            },
            entry: {
                [path_1.default.parse(config.output.filename).name]: path_1.default.join(opts.cwd, config.entry),
            },
            extraBabelPresets: config.extraBabelPresets,
            extraBabelPlugins: config.extraBabelPlugins,
            // configure library related options
            chainWebpack(memo) {
                memo.output.libraryTarget('umd');
                // modify webpack target
                if (config.platform === 'node') {
                    memo.target('node');
                }
                return memo;
            },
        });
    }
};
